package gregapi.cover.covers;

import static gregapi.data.CS.*;
import gregapi.cover.CoverData;
import gregapi.data.CS.SFX;
import gregapi.data.LH;
import gregapi.old.Textures;
import gregapi.render.BlockTextureDefault;
import gregapi.render.ITexture;
import gregapi.tileentity.connectors.MultiTileEntityPipeFluid;
import gregapi.tileentity.delegate.DelegatorTileEntity;
import gregapi.util.UT;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.entity.Entity;
import net.minecraft.item.ItemStack;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.ChunkCoordinates;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fluids.IFluidTank;

/**
 * @author Gregorius Techneticies
 */
public class CoverPressureValve extends AbstractCoverAttachmentController {
	@Override public boolean interceptCoverPlacement(byte aCoverSide, CoverData aData, Entity aPlayer) {return !(aData.mTileEntity instanceof MultiTileEntityPipeFluid);}
	
	@Override
	public void onTickPost(byte aSide, CoverData aData, long aTimer, boolean aIsServerSide, boolean aReceivedBlockUpdate, boolean aReceivedInventoryUpdate) {
		if (aIsServerSide && aData.mTileEntity instanceof MultiTileEntityPipeFluid) {
			ChunkCoordinates tCoords = aData.mTileEntity.getCoords();
			if ((aTimer % 2 == 0) != UT.Worlds.areCoordsEven(tCoords)) {
				IFluidTank tTank = ((MultiTileEntityPipeFluid)aData.mTileEntity).mTank;
		    	if (tTank.getFluidAmount() >= tTank.getCapacity() && tTank.getFluid().getFluid().isGaseous(tTank.getFluid())) {
					DelegatorTileEntity<IFluidHandler> tDelegator = aData.mTileEntity.getAdjacentTank(aSide);
					if (tDelegator.mTileEntity != null) {
						tTank.drain(tDelegator.mTileEntity.fill(tDelegator.getForgeSideOfTileEntity(), tTank.drain(Integer.MAX_VALUE, F), T), T);
					} else if (tDelegator.getAir(tDelegator.getCoords())) {
				    	UT.Sounds.send(aData.mTileEntity.getWorld(), SFX.MC_FIZZ, 1.0F, 1.0F, tCoords);
				    	try {for (Entity tEntity : (ArrayList<Entity>)aData.mTileEntity.getWorld().getEntitiesWithinAABB(Entity.class, AxisAlignedBB.getBoundingBox(tCoords.posX-2, tCoords.posY-2, tCoords.posZ-2, tCoords.posX+3, tCoords.posY+3, tCoords.posZ+3))) UT.Entities.applyTemperatureDamage(tEntity, tTank.getFluid().getFluid().getTemperature(tTank.getFluid()), 2.0F);} catch(Throwable e) {if (D1) e.printStackTrace(ERR);}
				    	tTank.drain(Integer.MAX_VALUE, T);
					}
		    	}
			}
		}
	}
	
	@Override
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		super.addToolTips(aList, aStack, aF3_H);
		aList.add(LH.Chat.CYAN + "Releases Gasses when Pipe is full.");
		aList.add(LH.Chat.ORANGE + "Requires Air or Tank in front!");
	}
	
	@Override public float[] getCoverBounds(byte aCoverSide, CoverData aData) {return BOXES_VALVES[aCoverSide];}
	@Override public float[] getHolderBounds(byte aCoverSide, CoverData aData) {return BOXES_VALVES[aCoverSide];}
	
	public static final float[][] BOXES_VALVES = new float[][] {{PIXELS_POS[ 6], PIXELS_POS[ 0], PIXELS_POS[ 6], PIXELS_NEG[ 6], PIXELS_NEG[ 8], PIXELS_NEG[ 6]}, {PIXELS_POS[ 6], PIXELS_POS[ 8], PIXELS_POS[ 6], PIXELS_NEG[ 6], PIXELS_NEG[ 0], PIXELS_NEG[ 6]}, {PIXELS_POS[ 6], PIXELS_POS[ 6], PIXELS_POS[ 0], PIXELS_NEG[ 6], PIXELS_NEG[ 6], PIXELS_NEG[ 8]}, {PIXELS_POS[ 6], PIXELS_POS[ 6], PIXELS_POS[ 8], PIXELS_NEG[ 6], PIXELS_NEG[ 6], PIXELS_NEG[ 0]}, {PIXELS_POS[ 0], PIXELS_POS[ 6], PIXELS_POS[ 6], PIXELS_NEG[ 8], PIXELS_NEG[ 6], PIXELS_NEG[ 6]}, {PIXELS_POS[ 8], PIXELS_POS[ 6], PIXELS_POS[ 6], PIXELS_NEG[ 0], PIXELS_NEG[ 6], PIXELS_NEG[ 6]}};
	
	@Override public ITexture getCoverTextureSurface(byte aSide, CoverData aData) {return sTextureFront;}
	@Override public ITexture getCoverTextureAttachment(byte aSide, CoverData aData, byte aTextureSide) {return aSide == aTextureSide ? sTextureFront : null;}
	@Override public ITexture getCoverTextureHolder(byte aSide, CoverData aData, byte aTextureSide) {return aSide == aTextureSide ? null : sTextureSide;}
	
	public static final ITexture
	sTextureFront = BlockTextureDefault.get(new Textures.BlockIcons.CustomIcon("machines/covers/pressurevalve/front")),
	sTextureSide = BlockTextureDefault.get(new Textures.BlockIcons.CustomIcon("machines/covers/pressurevalve/side"));
}