package gregapi.recipes.maps;

import static gregapi.data.CS.DW;
import static gregapi.data.CS.F;
import static gregapi.data.CS.GAPI_POST;
import static gregapi.data.CS.T;
import gregapi.code.ArrayListNoNulls;
import gregapi.data.IL;
import gregapi.data.LH;
import gregapi.gui.Slot_Normal;
import gregapi.gui.Slot_Whitelist;
import gregapi.item.IItemGTHandTool;
import gregapi.random.IHasWorldAndCoords;
import gregapi.recipes.ICraftingRecipeGT;
import gregapi.recipes.Recipe;
import gregapi.recipes.Recipe.RecipeMap;
import gregapi.tileentity.ITileEntityInventoryGUI;
import gregapi.util.OM;
import gregapi.util.UT;

import java.util.Collection;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraftforge.fluids.FluidStack;

/**
 * @author Gregorius Techneticies
 */
public class RecipeMapAutocrafting extends RecipeMap {
	public RecipeMapAutocrafting(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
		super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
	}
	
	public static final List<IRecipe> ALLOWED_RECIPES = new ArrayListNoNulls<IRecipe>();
	public static final List<IRecipe> RECENT_RECIPES = new ArrayListNoNulls<IRecipe>();
	
	@Override
	public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
		Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
		if (rRecipe != null || !IL.Paper_Blueprint_Used.equal(aSpecialSlot, F, T) || aInputs == null || aInputs.length < 1 || GAPI_POST.mFinishedServerStarted <= 0) return rRecipe;
		ItemStack[] tBlueprint = UT.NBT.getBlueprintCrafting(aSpecialSlot);
		if (tBlueprint.length <= 0) return null;
		for (ItemStack tPlan : tBlueprint) if (tPlan != null && tPlan.getItem() instanceof IItemGTHandTool) return null;
		
		if (ALLOWED_RECIPES.isEmpty()) {
			for (Object tCraftingRecipe : CraftingManager.getInstance().getRecipeList()) if (tCraftingRecipe instanceof IRecipe) {
				if (!(tCraftingRecipe instanceof ICraftingRecipeGT) || ((ICraftingRecipeGT)tCraftingRecipe).isAutocraftableByGT()) {
					ALLOWED_RECIPES.add((IRecipe)tCraftingRecipe);
				}
			}
		}
		
		InventoryCrafting tCraftInv = new InventoryCrafting(new Container() {@Override public boolean canInteractWith(EntityPlayer var1) {return F;}}, 3, 3);
		for (int i = 0; i < 9; i++) tCraftInv.setInventorySlotContents(i, tBlueprint[i]);
		
		IRecipe tIRecipe = null;
		
		for (int i = 0, j = RECENT_RECIPES.size(); i < j; i++) {
			if (RECENT_RECIPES.get(i).matches(tCraftInv, aTileEntity == null ? DW : aTileEntity.getWorld())) {
				tIRecipe = RECENT_RECIPES.get(i);
				break;
			}
		}
		
		if (tIRecipe == null) for (int i = 0, j = ALLOWED_RECIPES.size(); i < j; i++) {
			if (ALLOWED_RECIPES.get(i).matches(tCraftInv, aTileEntity == null ? DW : aTileEntity.getWorld())) {
				tIRecipe = ALLOWED_RECIPES.get(i);
				ALLOWED_RECIPES.remove(i);
				RECENT_RECIPES.add(tIRecipe);
				break;
			}
		}
		
		if (tIRecipe == null) return null;
		
		ItemStack tOutput = tIRecipe.getCraftingResult(tCraftInv);
		
		if (UT.Stacks.invalid(tOutput)) return null; 
		
		ArrayListNoNulls<ItemStack> tInputs = new ArrayListNoNulls<ItemStack>();
		ArrayListNoNulls<ItemStack> tOutputs = new ArrayListNoNulls<ItemStack>(F, tOutput);
		
		for (ItemStack tPlan : tBlueprint) if (tPlan != null) {
			boolean temp = T;
			for (ItemStack tInput : tInputs) if (UT.Stacks.equal(tInput, tPlan, F)) {
				tInput.stackSize++;
				tOutputs.add(UT.Stacks.container(tPlan, F));
				temp = F;
			}
			if (temp) {
				tInputs.add(UT.Stacks.amount(1, tPlan));
				tOutputs.add(UT.Stacks.container(tPlan, F));
			}
		}
		
		for (ItemStack tInput : tInputs) if (OM.is_("gt:autocrafterinfinite", tInput)) tInput.stackSize = 0;
		
		return new Recipe(T, F, tInputs.toArray(new ItemStack[tInputs.size()]), tOutputs.toArray(new ItemStack[tOutputs.size()]), null, null, null, null, 1024, 16, 0);
	}
	
	@Override
	public boolean containsInput(ItemStack aStack, ItemStack aSpecialSlot) {
		ItemStack[] tRecipe = UT.NBT.getBlueprintCrafting(aSpecialSlot);
		for (ItemStack tStack : tRecipe) if (UT.Stacks.equal(tStack, aStack, F)) return T;
		return super.containsInput(aStack, aSpecialSlot);
	}
	
	@Override
	public Slot_Normal getSpecialSlot(ITileEntityInventoryGUI aInventory, int aIndex, int aX, int aY) {
		return new Slot_Whitelist(aInventory, aIndex, aX, aY, IL.Paper_Blueprint_Used.get(1)).setTooltip(LH.AUTOCRAFTING_INSERT_BLUEPRINT, LH.Chat.WHITE);
	}
}
